@@ -22,12 +22,12 @@
 
 #include "lib/getopt.h"
 
-static char fio_version_string[] = "fio 1.53";
+static char fio_version_string[] = "fio 1.56";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
 static char **ini_file;
-static int max_jobs = MAX_JOBS;
+static int max_jobs = FIO_MAX_JOBS;
 static int dump_cmdline;
 
 static struct thread_data def_thread;
@@ -64,85 +64,90 @@ static char cmd_optstr[256];
  */
 static struct option l_opts[FIO_NR_OPTIONS] = {
 	{
-		.name		= "output",
+		.name		= (char *) "output",
 		.has_arg	= required_argument,
 		.val		= 'o',
 	},
 	{
-		.name		= "timeout",
+		.name		= (char *) "timeout",
 		.has_arg	= required_argument,
 		.val		= 't',
 	},
 	{
-		.name		= "latency-log",
+		.name		= (char *) "latency-log",
 		.has_arg	= required_argument,
 		.val		= 'l',
 	},
 	{
-		.name		= "bandwidth-log",
+		.name		= (char *) "bandwidth-log",
 		.has_arg	= required_argument,
 		.val		= 'b',
 	},
 	{
-		.name		= "minimal",
+		.name		= (char *) "minimal",
 		.has_arg	= optional_argument,
 		.val		= 'm',
 	},
 	{
-		.name		= "version",
+		.name		= (char *) "version",
 		.has_arg	= no_argument,
 		.val		= 'v',
 	},
 	{
-		.name		= "help",
+		.name		= (char *) "help",
 		.has_arg	= no_argument,
 		.val		= 'h',
 	},
 	{
-		.name		= "cmdhelp",
+		.name		= (char *) "cmdhelp",
 		.has_arg	= optional_argument,
 		.val		= 'c',
 	},
 	{
-		.name		= "showcmd",
+		.name		= (char *) "showcmd",
 		.has_arg	= no_argument,
 		.val		= 's',
 	},
 	{
-		.name		= "readonly",
+		.name		= (char *) "readonly",
 		.has_arg	= no_argument,
 		.val		= 'r',
 	},
 	{
-		.name		= "eta",
+		.name		= (char *) "eta",
 		.has_arg	= required_argument,
 		.val		= 'e',
 	},
 	{
-		.name		= "debug",
+		.name		= (char *) "debug",
 		.has_arg	= required_argument,
 		.val		= 'd',
 	},
 	{
-		.name		= "section",
+		.name		= (char *) "section",
 		.has_arg	= required_argument,
 		.val		= 'x',
 	},
 	{
-		.name		= "alloc-size",
+		.name		= (char *) "alloc-size",
 		.has_arg	= required_argument,
 		.val		= 'a',
 	},
 	{
-		.name		= "profile",
+		.name		= (char *) "profile",
 		.has_arg	= required_argument,
 		.val		= 'p',
 	},
 	{
-		.name		= "warnings-fatal",
+		.name		= (char *) "warnings-fatal",
 		.has_arg	= no_argument,
 		.val		= 'w',
 	},
+	{
+		.name		= (char *) "max-jobs",
+		.has_arg	= required_argument,
+		.val		= 'j',
+	},
 	{
 		.name		= NULL,
 	},
@@ -568,7 +573,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	}
 
 	if (profile_td_init(td))
-		return 1;
+		goto err;
 
 	engine = get_engine_name(td->o.ioengine);
 	td->io_ops = load_ioengine(td, engine);
@@ -771,7 +776,7 @@ static int is_empty_or_comment(char *line)
 			return 1;
 		if (line[i] == '#')
 			return 1;
-		if (!isspace(line[i]) && !iscntrl(line[i]))
+		if (!isspace((int) line[i]) && !iscntrl((int) line[i]))
 			return 0;
 	}
 
@@ -1039,6 +1044,7 @@ static void usage(const char *name)
 	printf("\t--alloc-size=kb\tSet smalloc pool to this size in kb"
 		" (def 1024)\n");
 	printf("\t--warnings-fatal Fio parser warnings are fatal\n");
+	printf("\t--max-jobs\tMaximum number of threads/processes to support\n");
 	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>");
 	printf("\n                   Jens Axboe <jaxboe@fusionio.com>\n");
 }
@@ -1222,10 +1228,8 @@ static int parse_cmd_line(int argc, char *argv[])
 
 			if (!strncmp(opt, "name", 4) && td) {
 				ret = add_job(td, td->o.name ?: "fio", 0);
-				if (ret) {
-					put_job(td);
+				if (ret)
 					return 0;
-				}
 				td = NULL;
 			}
 			if (!td) {
@@ -1249,6 +1253,14 @@ static int parse_cmd_line(int argc, char *argv[])
 		case 'w':
 			warnings_fatal = 1;
 			break;
+		case 'j':
+			max_jobs = atoi(optarg);
+			if (!max_jobs || max_jobs > REAL_MAX_JOBS) {
+				log_err("fio: invalid max jobs: %d\n", max_jobs);
+				do_exit++;
+				exit_val = 1;
+			}
+			break;
 		default:
 			do_exit++;
 			exit_val = 1;
@@ -1262,8 +1274,6 @@ static int parse_cmd_line(int argc, char *argv[])
 	if (td) {
 		if (!ret)
 			ret = add_job(td, td->o.name ?: "fio", 0);
-		if (ret)
-			put_job(td);
 	}
 
 	while (optind < argc) {
