@@ -22,9 +22,16 @@
 #define FIO_HAVE_POSIXAIO
 #define FIO_HAVE_CLOCK_MONOTONIC
 #define FIO_USE_GENERIC_RAND
+#define FIO_HAVE_GETTID
 
 #define OS_MAP_ANON		MAP_ANON
 
+/*
+ * OSX has a pitifully small shared memory segment by default,
+ * so default to a lower number of max jobs supported
+ */
+#define FIO_MAX_JOBS		128
+
 typedef off_t off64_t;
 
 /* OS X as of 10.6 doesn't have the timer_* functions. 
@@ -141,4 +148,9 @@ static inline unsigned long long os_phys_mem(void)
 	sysctl(mib, 2, &mem, &len, NULL, 0);
 	return mem;
 }
+
+static inline int gettid(void)
+{
+	return mach_thread_self();
+}
 #endif
